package mysql

import (
	"fmt"
	"sync"
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/configmdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	_ "github.com/go-sql-driver/mysql"

	"github.com/gocraft/dbr"
)

// Hold a single global connection (pooling provided by sql driver)
var sqlConnection *dbr.Connection
var sqlConnections map[string]*dbr.Connection
var connectionError error
var sqlOnce sync.Once
var config tomlConfig

// MYSQLConnection -
type mysqlConnection struct {
	Host            string        `json:"host" bson:"host"`
	Port            string        `json:"port" bson:"port"`
	Dbname          string        `json:"dbname" bson:"dbname"`
	Username        string        `json:"username" bson:"username"`
	Password        string        `json:"password " bson:"password "`
	Protocol        string        `json:"protocol" bson:"protocol"`
	Parameters      []string      `json:"params" bson:"params"`
	MaxIdleConns    int           `json:"maxIdleConns" bson:"maxIdleConns"`
	MaxOpenConns    int           `json:"maxOpenConns" bson:"maxOpenConns"`
	ConnMaxLifetime time.Duration `json:"connMaxLifetime" bson:"connMaxLifetime"`
}

type tomlConfig struct {
	MYSQLConnections map[string]mysqlConnection
}

// Init initializes MYSQL Connections for given toml file
func Init(tomlFilepath string) error {
	sqlOnce.Do(func() {
		sqlConnections = make(map[string]*dbr.Connection)
		_, err := configmdl.InitConfig(tomlFilepath, &config)
		if errormdl.CheckErr(err) != nil {
			loggermdl.LogError(err)
			connectionError = err
			return
		}
		for connectionName, connectionDetails := range config.MYSQLConnections {
			connection, err := dbr.Open("mysql", connectionDetails.Username+":"+connectionDetails.Password+"@"+connectionDetails.Protocol+"("+connectionDetails.Host+":"+connectionDetails.Port+")/"+connectionDetails.Dbname)
			if err != nil {
				sessionError = err
				return
			}
			fmt.Println(mysqlDSN.String())
			sqlConnections[connectionName] = connection
		}
	})
	return errormdl.CheckErr(sessionError)
}

//GetMYSQLConnection -
func GetMYSQLConnection(connectionName string) (*dbr.Connection, error) {
	if sqlConnections == nil {
		return nil, errormdl.Wrap("MYSQL_INIT_NOT_DONE")
	}
	if session, ok := sqlConnections[connectionName]; ok {
		return session, nil
	}
	return nil, errormdl.Wrap("Connection not found for instance: " + connectionName)
}

// GetMultipleSQLConnection -
func GetMultipleSQLConnection(tomlFilepath string) (map[string]*dbr.Connection, error) {
	sqlOnce.Do(func() {
		_, err := configmdl.InitConfig(tomlFilepath, &config)
		if errormdl.CheckErr(err) != nil {
			loggermdl.LogError(err)
			connectionError = err
			return
		}
		connectionsToReturn = make(map[string]*dbr.Connection)
		for connectionName, mysqlConnectionObj := range config.MYSQLConnections {
			connection, err := dbr.Open("mysql", mysqlConnectionObj.Username+":"+mysqlConnectionObj.Password+"@"+mysqlConnectionObj.Protocol+"("+mysqlConnectionObj.Host+":"+mysqlConnectionObj.Port+")/"+mysqlConnectionObj.Dbname, nil)
			if err != nil {
				connectionError = err
			}
			if mysqlConnectionObj.MaxIdleConns == 0 {
				connection.SetMaxIdleConns(100) // default is 2
			}
			if mysqlConnectionObj.MaxOpenConns == 0 {
				connection.SetMaxOpenConns(5000) // default there's no limit
			}
			if mysqlConnectionObj.ConnMaxLifetime == 0 {
				duration := 3 * 24 * time.Hour
				connection.SetConnMaxLifetime(duration)
			}
			sqlConnection = connection
			connectionsToReturn[connectionName] = sqlConnection
		}
	})
	return connectionsToReturn, errormdl.CheckErr(connectionError)
}
